package universum.ui;

import universum.engine.ScoreData;
import universum.bi.GameKind;
import java.util.*;

public final class GameResult {
    private List<ScoreData> results;
    private int numTurns;
    private GameKind gameKind;

    public GameResult(universum.engine.RenderingInfo ri) {
        init(ri.scores(), ri.numTurns(), ri.getGameKind());
    }

    public GameResult(Map<String,ScoreData> scores, 
                      int numTurns, GameKind gameKind) {
        init(scores, numTurns, gameKind);
    }

    private void init(Map<String,ScoreData> scores, 
                      int numTurns, GameKind gameKind) {
        results = new ArrayList<ScoreData>(scores.values());
        Collections.sort(results);
        this.numTurns = numTurns; 
        this.gameKind = gameKind;
    }
    

    public ScoreData winner() {
        if (results.size() > 0) {
            ScoreData rv = results.get(0);
            return rv.alive ? rv : null;
        }
        return null;
    }

    public List<ScoreData> results() {
        return this.results;
    }

    public String toString() {       
        StringBuffer result = new StringBuffer();
        result.append("gameKind: "+gameKind+"\n");
        result.append("numTurns: "+numTurns+"\n");        
        for (ScoreData sd: results) {            
            result.append(sd.player+" with id="+sd.id);
            result.append(":    jar="+sd.jarFile);
            result.append(":    score="+sd.score);
            result.append("     energy="+sd.energy);
            result.append("\n");
            String e = sd.getErrors();
            if (e.length() > 0) {
                result.append("Errors:\n");
                result.append(e);
            }
        }
        return result.toString();
    }

    public int numTurns() {
        return this.numTurns;
    }

    public GameKind kind() {
        return this.gameKind;
    }
}
